#ifndef _INTERFACE_H_
#define _INTERFACE_H_

/*
   *
   * interface.h
   *
   * Copyright (c) 2002-2003 Aleksander Augustowski
   *
   * BUG RAPORTS &
   * E-MAIL:          aleksander.augustowski@ins.com.pl
   * URL:             http://www.ins.com.pl/~alex/bcount/bcount.html
   *
   * This software is open source; you can redistribute it and/or modify
   * it under the terms of the GNU General Public License as published by
   * the Free Software Foundation; either version 2 of the License, or
   * (at your option) any later version.
   *
   * This program is distributed WITHOUT ANY WARRANTY; without even the
   * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
   * See the GNU General Public License in the included COPYING file for
   * details.
   *
   *
*/


#include <linux/if.h>
#include <netinet/ether.h>
#include <arpa/inet.h>
#include <linux/sockios.h>
#include "socket_stream.h"
#include "hwaddr_ntoa.h"
#include "interface_inet.h"
#include "interface_ltype.h"
#include "interface_desc.h"
#include "clear_buffer.h"

typedef unsigned long long u64;
typedef __uint32_t u32;        
typedef __uint16_t u16;        
typedef __uint8_t u8;          

#include "ethtool_copy.h"

#ifndef SIOCETHTOOL
#define  SIOCETHTOOL	0x8946
#endif

#define SLEEP_TIME		1

#define KBYTES_PER_SEC		"KB/s"
#define BYTES_PER_SEC		"B/s"

#define TEN_MBYTES_PER_SEC	"10Mb/s"
#define HUNDRED_MBYTES_PER_SEC	"100Mb/s"
#define THOUSAND_MBYTES_PER_SEC	"1000Mb/s"

#define ETHERNET_FULL_DUPLEX	"Full"
#define ETHERNET_HALF_DUPLEX	"Half"

#define UP_STRING		"UP"
#define DOWN_STRING		"DN"

#define IPV4SIZE		16
#define DESCSIZE		20
#define HWSIZE			32

/* Address to accept any incoming messages. */
#ifndef	INADDR_ANY /* 0.0.0.0 */
#define	INADDR_ANY	((unsigned long int) 0x00000000)
#endif

/* Address to loopback in software to local host.  */
#ifndef	INADDR_LOOPBACK /* 127.0.0.1 */
#define	INADDR_LOOPBACK		0x7f000001
#endif

/* Heart for bcount project :-) */
#define PROC_NET_DEV		"/proc/net/dev"

/* Nobody have 666 interfaces :-) installed in system
    and it not equal with return integer with search_for_interface() */
#define	INTERFACE_NOT_FOUND	666

/* Default interface name lenght */
#ifndef	IFNAMESIZE
#define	IFNAMESIZE		16
#endif

#define	STATSIZE		512

/* Max. count of Interface in system */
#ifndef	MAXIFN
#define	MAXIFN			IFNAMESIZE
#endif

/* IPv4 Octets definition */
#define IPOCTET_1		1
#define IPOCTET_2		2
#define IPOCTET_3		3
#define IPOCTET_4		4
				
#define HW_NULL_ADDRESS		"0:0:0:0:0:0"
#define NOT_AVAILABLE		"N/A"


void interface_not_found(char *ife);
void devices_file_error(void);
void free_interface_list(char **ifs_list);
void proc_net_dev_read_error(void);
int search_for_interface(char *interface);
unsigned int get_proc_file_size(char *file_patch);
char* fill_buffer_from_file(char *file_patch);
void get_interface_stats(char *interface);
int get_interfaces_amount(void);
int get_current_interface_number(char *interface);
char **get_interfaces_names(void);
char* get_interface_name_by_index(int index);
char* get_interfaces_list(void);
int get_ip_octets(char *interface_ip, int octet_number);
char get_ip_class(char *interface, char *interface_ip);
char* get_interface_ip(char *interface);
char* get_interface_dip(char *interface); /* For PtoP interfaces */
char* get_interface_netmask(char *interface);
char* get_interface_broadcast(char *interface);
char* get_interface_hw_address(char *interface);
int get_interface_mtu(char *interface);
int get_interface_metric(char *interface);
int get_interface_flags(char *interface);
char* get_interface_flags_name(char *interface);
unsigned int get_interface_hw_memstart(char *interface);
unsigned int get_interface_hw_memend(char *interface);
unsigned int get_interface_hw_base_addr(char *interface);
int get_interface_hw_irq(char *interface);
int get_interface_hw_dma(char *interface);
int get_interface_encap(char *interface);
char* get_ethernet_bus_info(char *interface);
char* get_ethernet_firmware(char *interface);
char* get_ethernet_driver_name(char *interface);
char* get_ethernet_driver_version(char *interface);
char* get_ethernet_link_detect(char *interface);
char* get_ethernet_speed(char *interface);
char* get_ethernet_duplex(char *interface);
void reset_interface_struct_stats (void);
void* interface_stats_read (void* unused);
void* interface_speed_measure (void* unused);
#ifdef _BCOUNT_H_
void reset_speed_window(void);
extern void deactivate_leds(void);
extern void activate_leds(void);
#endif


char default_interface_name[IFNAMESIZE] = ETHERNET"0";	/* eth0 :-) */
int default_interface_ltype = LTYPE_ETHERNET;
char default_interface_desc[20] = DESC_ETHERNET;	/* Ethernet */
unsigned short int clear_interface_speed_flag = FALSE;
unsigned short int freed_interface_data_flag = TRUE;
unsigned short int freed_interfaces_in_system_flag = TRUE;
static char not_available[] = NOT_AVAILABLE;
struct in_addr ia;		/* Internet address */
struct ifreq *ifr, ifre;	/* Interface ifreq struct pointer buffer */
struct ifconf *ifc, ifcn;	/* Interface configuration pointer buffer */
struct ifmap hwsetup;		/* DMA, IRQ, Base address */
struct ethtool_drvinfo drv;	/* Ethernet driver info */
struct ethtool_value val;	/* Struct for passing single values */
struct ethtool_cmd ecmd;	/* Settings: Speed, duplex... */



/* Struct of interface stats */
struct interface_statistics
{
	/* Received data */
	unsigned long long rx_bytes;
	double rx_kbytes;
	double rx_mbytes;
	unsigned long long rx_packets;
	unsigned long long rx_packets_last;
	unsigned long rx_errs;
	unsigned long rx_drop;
	unsigned long rx_fifo;
	unsigned long rx_frame;
	unsigned long rx_compressed; 
	unsigned long rx_multicast;
	/* Transmitted data */
	unsigned long long tx_bytes;
	double tx_kbytes;
	double tx_mbytes;
	unsigned long long tx_packets;
	unsigned long long tx_packets_last;
	unsigned long tx_errs;
	unsigned long tx_drop;
	unsigned long tx_fifo;
	unsigned long tx_colls;
	unsigned long tx_carrier;
	unsigned long tx_compressed;
} __attribute__ ((packed));


/* Interface speed (packets) */
struct interface_packets_speed
{
	/* TX veriables (packets) */
	unsigned long long now_tx_packets;
	unsigned long long last_tx_packets;
	/* RX veriables (packets) */
	unsigned long long now_rx_packets;
	unsigned long long last_rx_packets;
	/* TX speed_current_tx_packets per sec. = 
	    (now_tx_packets - last_tx_packets) 
	    and speed_current_tx_packets_top */
	unsigned long long speed_current_tx_packets;
	unsigned long long speed_current_tx_packets_top;
	/* RX speed_current_rx_packets per sec. = 
	    (now_rx_packets - last_rx_packets) 
	    and speed_current_rx_packets_top */
	unsigned long long speed_current_rx_packets;
	unsigned long long speed_current_rx_packets_top;
	/* TX & RX speed_current_packets per sec. = 
	    ((now_tx_packets - last_tx_packets) 
	    + (now_rx_packets - last_tx_packets)) 
	    and speed_packets_top which is > speed_current_packets */
	unsigned long long speed_current_packets; /* Packets Speed per sec. on an Interface */
	unsigned long long speed_packets_top;	  /* Top Packets speed per sec. on an Interface */
} __attribute__ ((packed));


/* Interface speed (KB/s) */
struct interface_kbytes_speed
{
	/* TX veriables (kbytes) */
	double now_tx_kbytes;
	double last_tx_kbytes;
	/* RX veriables (kbytes) */
	double now_rx_kbytes;
	double last_rx_kbytes;
	/* TX speed_current_tx_kbytes in KB/s = (now_tx_kbytes - last_tx_kbytes) 
	    and speed_current_tx_kbytes_top */
	double speed_current_tx_kbytes;
	double speed_current_tx_kbytes_top;
	/* RX speed_current_rx in KB/s = (now_rx_kbytes - last_rx_kbytes) 
	    and speed_current_rx_top */
	double speed_current_rx_kbytes;
	double speed_current_rx_kbytes_top;
	/* TX & RX speed_current_kbytes in KB/s = 
	    ((now_tx_kbytes - last_tx_kbytes) + (now_rx_kbytes - last_tx_rbytes)) 
	    and speed_top which is > speed_current */
	double speed_current_kbytes;	/* Speed in KB/s on an Interface */
	double speed_kbytes_top;	/* Top speed in KB/s on an Interface */
} __attribute__ ((packed));


/* Interface speed */
struct interface_speed
{
	struct interface_packets_speed pss;
	struct interface_kbytes_speed kbs;
} __attribute__ ((packed));


/* Ethernet Interface struct if_speed */
struct ethernet_driver_info
{
	/* Ethernet info (eth%d) */
        char driver[HWSIZE];		/* Driver name ex: 3c59x, eepro100... */
        char driver_version[HWSIZE];	/* Driver version */
        char firmware[HWSIZE];		/* FirmWare (if imlemented) */
        char bus_info[HWSIZE];		/* Interface Bus info ex. 00:0c.0 */
        char speed[10];			/* Eth speed: 10Mb/s, 100Mb/s ... */
        char duplex[10];		/* Eth duplex: half, full */
        char link[5];			/* Link detect: yes or no */
	/* end Ethernet info */
} __attribute__ ((packed));


/* Names and quantity interfaces installed in system */
typedef struct interfaces_in_system
{
	char interfaces_names[MAXIFN][IFNAMESIZE];	/* Interfaces names 
							    available in 
							    system */
	int interfaces_counter;				/* Interfaces counter 
							    available in 
							    system */
} Interfaces;

//Interfaces infes, *is = &infes;
//Interfaces *is;
struct interfaces_in_system *is;


/* Setup & information about interface */
typedef struct interface_info
{
	char name[IFNAMESIZE];		/* Current Interface name */
	int interface_number;		/* Current Interface number */
        int flags; 			/* Flags integer ...IFF_UP, etc. */
        int ltype; 			/* Link type ...PtoP, Ethernet, ... */
        char description[DESCSIZE];	/* Encap string ... Fiber, Ethernet */
        char ip_addr[IPV4SIZE];		/* IP address */
        char ip_daddr[IPV4SIZE];	/* IP address for PtoP interface */
        char ip_addr_class;		/* IP address class */
        char ip_daddr_class;		/* IP address for PtoP interface class */
        char netmask[IPV4SIZE];		/* Interface netmask */
        char broadcast[IPV4SIZE];	/* Interface broadcast */
        char hw_addr[HWSIZE];		/* HW address (MAC) xx:xx:xx:xx:xx:xx */
        char nflags[500];		/* Flags names string */
        int metric; 			/* Metric value */
        int mtu; 			/* Mtu value ex. ...1500 Bytes */
        int irq; 			/* IRQ int number */
        int dma; 			/* DMA int number */
        unsigned int mem_start;		/* Start Mem address display in hex */
        unsigned int mem_end;		/* End Mem address display in hex */
        unsigned int base_addr;		/* Base address display in hex */
	struct ethernet_driver_info eth;	/* Ethernet driver name ... */
	struct interface_speed speed;		/* Speed on an Interface */
        struct interface_statistics stats;	/* Statistic for interface */
} Interface;

//Interface infe, *i = &infe;
//Interface *i;
struct interface_info *i;


void interface_not_found(char *ife)
{
	printf ("Interface "_STRING_WHITE_" not found, exiting...\n", ife);

	exit_prog(EXIT_FALSE);
}


void devices_file_error(void)
{
	printf ("File "_STRING_WHITE_" not found.\nThis program don't execute "
		"without \"proc file system\", exiting...\n", PROC_NET_DEV);

	exit_prog(EXIT_FALSE);
}


void proc_net_dev_read_error(void)
{
	printf ("I can't read file "_STRING_WHITE_" into memory.\n"
		"This program don't execute if I can't read this file!, "
		"exiting...\n", PROC_NET_DEV);

	exit_prog(EXIT_FALSE);
}


void free_interfaces_list(char **ifes_list)
{
	int cnt = 0;
	
	while (cnt != is->interfaces_counter)
	    {
		free (ifes_list[cnt]);
		cnt++;
	    }
}


void get_interfaces_in_system(void)
{
	int ife_counter = 0;

	is->interfaces_counter = get_interfaces_amount();

	while (ife_counter != is->interfaces_counter)
	    {
		/*strncpy (is->interfaces_names[ife_counter], 
			get_interface_name_by_index(ife_counter),
			sizeof(get_interface_name_by_index(ife_counter)));
                */
	        strcpy (is->interfaces_names[ife_counter], 
			get_interface_name_by_index(ife_counter) );

		ife_counter++;
	    }

	freed_interfaces_in_system_flag = FALSE;
}


void free_interface_info_struct(void)
{
	if (freed_interface_data_flag == FALSE)
	    {
		free(i);
		freed_interface_data_flag = TRUE;
	    }
}


void free_interfaces_in_system_struct(void)
{
	if (freed_interfaces_in_system_flag == FALSE)
	    {
		free(is);
		freed_interface_data_flag = TRUE;
	    }
}


void set_default_interface_as_loopback(void)
{
	strncpy (default_interface_name, 
		LOOPBACK, sizeof(LOOPBACK));		/* lo */

	default_interface_ltype = LTYPE_LOOPBACK;

	strncpy (default_interface_desc, 
		DESC_LOOPBACK, sizeof(DESC_LOOPBACK));	/* Loopback */
}


void get_interface_data(char *ife)
{
	if (search_for_interface(ife) == INTERFACE_NOT_FOUND)
	    interface_not_found(ife);

	strncpy (i->name, ife, sizeof(i->name));

	i->interface_number = get_current_interface_number(i->name);
	i->flags = get_interface_flags(i->name);
	i->ltype = get_interface_ltype(i->name);
	strcpy (i->description, get_interface_desc(i->name));
	strcpy (i->ip_addr, get_interface_ip(i->name));
	i->ip_addr_class = get_ip_class(i->name, i->ip_addr);
	if (i->ltype == LTYPE_PPP || i->ltype == LTYPE_IPPP)
	    {
		strcpy (i->ip_daddr, get_interface_dip(i->name));
		i->ip_daddr_class = get_ip_class(i->name, i->ip_daddr);
	    }	    
	else
	    {
		strcpy (i->ip_daddr, not_available);
		i->ip_daddr_class = '?';
	    }
	strcpy (i->netmask, get_interface_netmask(i->name));
	strcpy (i->broadcast, get_interface_broadcast(i->name));
	strcpy (i->hw_addr, get_interface_hw_address(i->name));
	i->mtu = get_interface_mtu(i->name);
	i->metric = get_interface_metric(i->name);
	strcpy (i->nflags, get_interface_flags_name(i->name));
	i->irq = get_interface_hw_irq(i->name);
	i->dma = get_interface_hw_dma(i->name);
	i->mem_start = get_interface_hw_memstart(i->name);
	i->mem_end = get_interface_hw_memend(i->name);
	i->base_addr = get_interface_hw_base_addr(i->name);
	if (i->ltype == LTYPE_ETHERNET)
	    {
		strcpy (i->eth.driver, get_ethernet_driver_name(i->name));
		strcpy (i->eth.driver_version, get_ethernet_driver_version(i->name));
		strcpy(i->eth.firmware, not_available);
		strcpy (i->eth.bus_info, get_ethernet_bus_info(i->name));
		strcpy (i->eth.speed, get_ethernet_speed(i->name));
		strcpy (i->eth.duplex, get_ethernet_duplex(i->name));
		strcpy (i->eth.link, get_ethernet_link_detect(i->name));
	    }
	else
	    {
		strcpy(i->eth.driver, not_available);
		strcpy(i->eth.driver_version, not_available);
		strcpy(i->eth.firmware, not_available);
		strcpy(i->eth.bus_info, not_available);
		strcpy(i->eth.speed, not_available);
		strcpy(i->eth.duplex, not_available);
		strcpy(i->eth.link, not_available);
	    }

	freed_interface_data_flag = FALSE;
}


void get_interface_changeable_data(char *ife)
{
	if (search_for_interface(ife) == INTERFACE_NOT_FOUND)
	    interface_not_found(ife);

	i->flags = get_interface_flags(i->name);
	strcpy (i->ip_addr, get_interface_ip(i->name));
	i->ip_addr_class = get_ip_class(i->name, i->ip_addr);
	if (i->ltype == LTYPE_PPP || i->ltype == LTYPE_IPPP)
	    {
		strcpy (i->ip_daddr, get_interface_dip(i->name));
		i->ip_daddr_class = get_ip_class(i->name, i->ip_daddr);
	    }	    
	else
	    {
		strcpy (i->ip_daddr, not_available);
		i->ip_daddr_class = '?';
	    }
	strcpy (i->netmask, get_interface_netmask(i->name));
	strcpy (i->broadcast, get_interface_broadcast(i->name));
	strcpy (i->hw_addr, get_interface_hw_address(i->name));
	i->mtu = get_interface_mtu(i->name);
	i->metric = get_interface_metric(i->name);
	strcpy (i->nflags, get_interface_flags_name(i->name));
	if (i->ltype == LTYPE_ETHERNET)
	    {
		strcpy (i->eth.speed, get_ethernet_speed(i->name));
		strcpy (i->eth.duplex, get_ethernet_duplex(i->name));
		strcpy (i->eth.link, get_ethernet_link_detect(i->name));
	    }
	else
	    {
		strcpy(i->eth.speed, not_available);
		strcpy(i->eth.duplex, not_available);
		strcpy(i->eth.link, not_available);
	    }

	freed_interface_data_flag = FALSE;
}


char **get_interfaces_names(void)
{
	void *buffer, *pointer;
	static char **interfaces_list;
	int length, counter = 0, last_length;
	int ifreq_size = sizeof(struct ifreq);

	inet_socket = inet_socket_stream("get_interfaces_names()");

	last_length = 0;
	length = MAXIFN * ifreq_size;

	while(1)  
	    {
		buffer = xmalloc(length);

		ifcn.ifc_buf = buffer;
		ifcn.ifc_len = length;

		if (ioctl (inet_socket, SIOCGIFCONF, &ifcn) < 0)
		    {
			if (errno == EINVAL || last_length == 0)
			    {
				fprintf(stderr, _STRING_RED_
					": get_interfaces_names() -> ioctl(SIOCGIFCONF)\n", 
					strerror(errno));
				
				close_inet_socket_stream();

				exit_prog(EXIT_FALSE);
			    }
		    }
		else
		    {
			if (ifcn.ifc_len == last_length)
			    {
				free(buffer);
				break;
			    }

			last_length = ifcn.ifc_len;
		    }

		length += 10 * ifreq_size;
		free(buffer);
	    }
	
	length = ifcn.ifc_len;

	buffer = (char *) malloc (length);
	strncpy (buffer, ifcn.ifc_buf, sizeof(ifcn.ifc_buf) - 1);

	pointer = buffer;

	interfaces_list = (char **) xmalloc(sizeof(char *) 
			    * (length/ifreq_size +1));

	while (pointer < buffer + length)
	    {
		ifr = (struct ifreq *) pointer;	
		pointer += ifreq_size;	

		interfaces_list[counter] = (char *) xmalloc (sizeof(char) 
			* strlen(ifr->ifr_name));

		strncpy(interfaces_list[counter], 
			ifr->ifr_name, sizeof(ifr->ifr_name) - 1);

		counter++;
	    }
	
	interfaces_list[counter] = NULL;
	
	free(buffer);
	free(pointer);

	close_inet_socket_stream();

	return(interfaces_list);
}


int get_current_interface_number(char *interface)
{
	int ifnr = 0;	/* Interface number */

	ifr = (struct ifreq *) xmalloc (sizeof (struct ifreq));

	inet_socket = inet_socket_stream("get_current_interface_number()");

	strcpy (ifr->ifr_name, interface);
	ifr->ifr_addr.sa_family = PF_INET;

	if ((ioctl (inet_socket, SIOCGIFINDEX, ifr)) < 0)
	    {
		close_inet_socket_stream();
		free (ifr);

		return(-1);
	    }
	
	ifnr = ifr->ifr_ifindex;

	close_inet_socket_stream();
	free (ifr);

	return ifnr;
}


/* Get number/quantity interfaces instaled in system */
int get_interfaces_amount(void)
{
	struct ifreq ifr[MAXIFN];
	int inr = 0;	/* Interface number counter */
		
	ifc = (struct ifconf *) xmalloc (sizeof (struct ifconf));

	inet_socket = inet_socket_stream("get_interfaces_amount()");

	ifc->ifc_len = MAXIFN * sizeof(struct ifreq);
	ifc->ifc_req = ifr;
	
	if ((ioctl (inet_socket, SIOCGIFCONF, ifc)) < 0)
	    {
		close_inet_socket_stream();
		free (ifc);

		return(-1);
	    }

	inr = (int) (ifc->ifc_len / sizeof(struct ifreq));

	close_inet_socket_stream();
	free (ifc);

	return inr;
}


char* get_interface_name_by_index(int index)
{
	struct ifreq ifr[MAXIFN];
	static char interface_name[IFNAMESIZE];
	int rows = 0;
		
	ifc = (struct ifconf *) xmalloc (sizeof (struct ifconf));

	inet_socket = inet_socket_stream("get_interface_name_by_index()");

	ifc->ifc_len = MAXIFN * sizeof(struct ifreq);
	ifc->ifc_req = ifr;
	
	/* Add NULL char in all size of hardware_list because it is static */
	rows = 0;
	while (rows != ifc->ifc_len) {
		interface_name[rows]= '\0';
		rows++;
	    }

	if ((ioctl (inet_socket, SIOCGIFCONF, ifc)) < 0) {
		close_inet_socket_stream();
		free (ifc);

		return NULL;
	    }
	
	if (index < ifc->ifc_len / sizeof (struct ifreq)) 
	    {
	    	strncpy (interface_name, ifr[index].ifr_name,
			sizeof(ifr[index].ifr_name) - 1) ;
	        interface_name[sizeof(ifr[index].ifr_name) - 1] = '\0';
	    }
	else
	    {
		close_inet_socket_stream();
		free (ifc);

		return NULL;
	    }
	
	close_inet_socket_stream();
	free (ifc);
	
	return interface_name;
}


char* get_interfaces_list(void)
{
	struct ifreq ifr[MAXIFN];
	static char hardware_list[MAXIFN * sizeof(struct ifreq)];
	int number = 0;
	
	ifc = (struct ifconf *) xmalloc (sizeof (struct ifconf));

	inet_socket = inet_socket_stream("get_interface_list()");

	ifc->ifc_len = MAXIFN * sizeof(struct ifreq);
	ifc->ifc_req = ifr;
	
	while (number != ifc->ifc_len)
	    {
		hardware_list[number] = '\0';
		number++;
	    }

	if ((ioctl (inet_socket, SIOCGIFCONF, ifc)) < 0)
	    {
		close_inet_socket_stream();
		free (ifc);

		return NULL;
	    }
	
	for (number = 0; number < ifc->ifc_len / sizeof (struct ifreq); number++)
	    {
		if (number == 0)
		    strcpy (hardware_list, ifr[number].ifr_name);
		else
		    strcat (hardware_list, ifr[number].ifr_name);

		strcpy(is->interfaces_names[number], ifr[number].ifr_name);
		
		hardware_list[strlen(hardware_list)] = ' ';
	    }
	
	hardware_list[strlen(hardware_list)] = '\0';
	    
	close_inet_socket_stream();
	free (ifc);

	return hardware_list;
}


int get_ip_octets(char *interface_ip, int octet_number)
{
	char ip_address[strlen(interface_ip)];
	int ip_octet1, ip_octet2, ip_octet3, ip_octet4;

	strcpy (ip_address, interface_ip);

	sscanf (ip_address, "%d.%d.%d.%d", &ip_octet1, &ip_octet2, 
		&ip_octet3, &ip_octet4);

	if (octet_number == IPOCTET_1)
	    return ip_octet1;
	else if (octet_number == IPOCTET_2)
	    return ip_octet2;
	else if (octet_number == IPOCTET_3)
	    return ip_octet3;
	else if (octet_number == IPOCTET_4)
	    return ip_octet4;
	
	return ip_octet1;
}


char get_ip_class(char *interface, char *interface_ip)
{
	unsigned msb; /* Important byte */
	struct sockaddr_in adr_inet;
	int ip_octet1 = get_ip_octets(interface_ip, IPOCTET_1);
	int ip_octet2 = get_ip_octets(interface_ip, IPOCTET_2);
	int ip_octet3 = get_ip_octets(interface_ip, IPOCTET_3);
	int ip_octet4 = get_ip_octets(interface_ip, IPOCTET_4);
	int len_inet;
	char ip_class; /* Internet Protocol address class: A, B, C, D, E */
	struct
	    {
		unsigned char ip[4];
	    } ip_octets[] = 
		    {
			{ { ip_octet1, ip_octet2, ip_octet3, ip_octet4 } }
		    };
	
	if (interface_ip == NULL)
	    return '?';

	memset (&adr_inet, 0, sizeof(adr_inet));
	adr_inet.sin_family = PF_INET;
	memcpy (&adr_inet.sin_addr.s_addr, ip_octets[0].ip, 4);
	len_inet = sizeof(adr_inet);
		
	msb = *(unsigned char *) &adr_inet.sin_addr.s_addr;

	if ((msb & 0x80) == 0x00)
	    ip_class = 'A';
	else if ((msb & 0xC0) == 0x80)
	    ip_class = 'B';
	else if ((msb & 0xE0) == 0xC0)
	    ip_class = 'C';
	else if ((msb & 0xF0) == 0xE0)
	    ip_class = 'D';
	else
	    ip_class = 'E';
	
	return ip_class;
}


char* get_interface_ip(char *interface)
{
	static char ip_address[IPV4SIZE];

	ifr = (struct ifreq *) xmalloc (sizeof (struct ifreq));

	inet_socket = inet_socket_stream("get_interface_ip()");

	strcpy (ifr->ifr_name, interface);
	ifr->ifr_addr.sa_family = PF_INET;

	/* Get interface IP address (IPv4) */
	if ((ioctl (inet_socket, SIOCGIFADDR, ifr)) < 0)
	    {
		close_inet_socket_stream();
		free (ifr);

		return NULL;
	    }
	else
	    memcpy ((char *) &ia, ifr->ifr_addr.sa_data + 2, 4);

	strcpy (ip_address, inet_ntoa(ia));

	close_inet_socket_stream();
	free (ifr);
    
	return ip_address;
}


char* get_interface_dip(char *interface)
{
	static char dip_address[IPV4SIZE];
	int flags = get_interface_flags(interface);

	ifr = (struct ifreq *) xmalloc (sizeof (struct ifreq));

	inet_socket = inet_socket_stream("get_interface_dip()");

	strcpy (ifr->ifr_name, interface);
	ifr->ifr_addr.sa_family = PF_INET;

	/* If interface is P-to-P then we get destination IP address */
	if (flags & IFF_POINTOPOINT)
	    {
		/* Get interface IP destination address (IPv4) */
		if ((ioctl (inet_socket, SIOCGIFDSTADDR, ifr)) < 0)
		    {
			close_inet_socket_stream();
			free (ifr);

			return NULL;
		    }
		else
		    memcpy ((char *) &ia, ifr->ifr_dstaddr.sa_data + 2, 4);
	    }
	else
	    {
		close_inet_socket_stream();
		free (ifr);

		return NULL;
	    }

	strcpy (dip_address, inet_ntoa(ia));

	close_inet_socket_stream();
	free (ifr);

	return dip_address;
}


char* get_interface_netmask(char *interface)
{
	static char netmask[IPV4SIZE];
	
	ifr = (struct ifreq *) xmalloc (sizeof (struct ifreq));

	inet_socket = inet_socket_stream("get_interface_netmask()");

	strcpy (ifr->ifr_name, interface);
	ifr->ifr_netmask.sa_family = PF_INET;

	/* Get interface Network mask */
	if ((ioctl (inet_socket, SIOCGIFNETMASK, ifr)) < 0)
	    {
		close_inet_socket_stream();
		free (ifr);

		return NULL;
	    }
	else
	    memcpy ((char *) &ia, ifr->ifr_netmask.sa_data + 2, 4);

	strcpy (netmask, inet_ntoa(ia));

	close_inet_socket_stream();
	free (ifr);

	return netmask;
}


char* get_interface_broadcast(char *interface)
{
	static char broadcast[IPV4SIZE];
	int flags = get_interface_flags(interface);

	if (!(flags & IFF_BROADCAST))
	    return not_available;

	ifr = (struct ifreq *) xmalloc (sizeof (struct ifreq));

	inet_socket = inet_socket_stream("get_interface_broadcast()");

	strcpy (ifr->ifr_name, interface);
	ifr->ifr_broadaddr.sa_family = PF_INET;

	/* Get interface Broadcast address */
	if ((ioctl (inet_socket, SIOCGIFBRDADDR, ifr)) < 0)
	    {
		close_inet_socket_stream();
		free (ifr);

		return NULL;
	    }
	else
	    memcpy ((char *) &ia, ifr->ifr_broadaddr.sa_data + 2, 4);

	strcpy (broadcast, inet_ntoa(ia));

	close_inet_socket_stream();
	free (ifr);

	return broadcast;
}


char* get_interface_hw_address(char *interface)
{
	struct ether_addr *mac_address;

	ifr = (struct ifreq *) xmalloc (sizeof (struct ifreq));

	inet_socket = inet_socket_stream("get_interface_hw_address()");

	strcpy (ifr->ifr_name, interface);
	ifr->ifr_hwaddr.sa_family = PF_INET;

	/* Get interface HW (MAC) address */
	if ((ioctl (inet_socket, SIOCGIFHWADDR, ifr)) < 0)
	    {
		close_inet_socket_stream();
		free (ifr);

		return NULL;
	    }
	else
	    ((char *) mac_address) = ifr->ifr_hwaddr.sa_data;

	close_inet_socket_stream();
	free (ifr);

	if (!strcmp ((ether_ntoa(mac_address)), HW_NULL_ADDRESS))
	    return not_available;
	    
	/* return ether_ntoa(mac_address); */
	return hwaddr_ntoa(mac_address->ether_addr_octet);
}


int get_interface_mtu(char *interface)
{
	int mtu_value = 0;

	ifr = (struct ifreq *) xmalloc (sizeof (struct ifreq));

	inet_socket = inet_socket_stream("get_interface_mtu()");

	strcpy (ifr->ifr_name, interface);
	/* Get interface MTU value */
	if ((ioctl (inet_socket, SIOCGIFMTU, ifr)) < 0)
	    mtu_value = 0;
	else
	    mtu_value = ifr->ifr_mtu;

	close_inet_socket_stream();
	free (ifr);

	return mtu_value;
}


int get_interface_metric(char *interface)
{
	int metric_value = 0;

	ifr = (struct ifreq *) xmalloc (sizeof (struct ifreq));

	inet_socket = inet_socket_stream("get_interface_metric()");

	strcpy (ifr->ifr_name, interface);
	/* Get interface Metric value */
	if ((ioctl (inet_socket, SIOCGIFMETRIC, ifr)) < 0)
	    metric_value = 0;
	else
	    metric_value = ifr->ifr_metric;

	close_inet_socket_stream();
	free (ifr);

	return metric_value ? metric_value : 1;
}


int get_interface_flags(char *interface)
{
	int interface_flags = 0;

	ifr = (struct ifreq *) xmalloc (sizeof (struct ifreq));

	inet_socket = inet_socket_stream("get_interface_flags()");

	strcpy (ifr->ifr_name, interface);
	/* Get interface Flags */
	if ((ioctl (inet_socket, SIOCGIFFLAGS, ifr)) < 0)
	    interface_flags = 0;
	else
	    interface_flags = ifr->ifr_flags;

	close_inet_socket_stream();
	free (ifr);

	return interface_flags;
}


char* get_interface_flags_name(char *interface)
{
	static char interface_flags[200];
	int flags = get_interface_flags(interface);

	if (flags == 0)
	    {
		strcpy (interface_flags, "NO FLAGS!\n");
		interface_flags[strlen(interface_flags)] = '\0';
		return interface_flags;
	    }

	if (flags & IFF_UP)
	    strcpy (interface_flags, "UP");

	if (flags & IFF_BROADCAST)
	    strcat (interface_flags, " BROADCAST");

	if (flags & IFF_DEBUG)
	    strcat (interface_flags, " DEBUG");

	if (flags & IFF_LOOPBACK)
	    strcat (interface_flags, " LOOPBACK");

	if (flags & IFF_POINTOPOINT)
	    strcat (interface_flags, " POINTOPOINT");

	if (flags & IFF_NOTRAILERS)
	    strcat (interface_flags, " NOTRAILERS");

	if (flags & IFF_RUNNING)
	    strcat (interface_flags, " RUNNING");

	if (flags & IFF_NOARP)
	    strcat (interface_flags, " NOARP");

	if (flags & IFF_PROMISC)
	    strcat (interface_flags, " PROMISC");

	if (flags & IFF_ALLMULTI)
	    strcat (interface_flags, " ALLMULTI");

	if (flags & IFF_SLAVE)
	    strcat (interface_flags, " SLAVE");

	if (flags & IFF_MASTER)
	    strcat (interface_flags, " MASTER");

	if (flags & IFF_MULTICAST)
	    strcat (interface_flags, " MULTICAST");

	if (flags & IFF_DYNAMIC)
	    strcat (interface_flags, " DYNAMIC");

	interface_flags[strlen(interface_flags)] = '\0';

	return interface_flags;
}


unsigned int get_interface_hw_memstart(char *interface)
{
	ifr = (struct ifreq *) xmalloc (sizeof (struct ifreq));

	inet_socket = inet_socket_stream("get_interface_hw_memstart()");

	strcpy (ifr->ifr_name, interface);
	if ((ioctl (inet_socket, SIOCGIFMAP, ifr)) < 0)
	    memset (&hwsetup, 0, sizeof(struct ifmap));
	else
	    memcpy (&hwsetup, &ifr->ifr_map, sizeof(struct ifmap));

	strcpy (ifr->ifr_name, interface);
	if ((ioctl (inet_socket, SIOCGIFMAP, ifr)) < 0)
	    memset (&hwsetup, 0, sizeof(struct ifmap));
	else
	    hwsetup = ifr->ifr_map;

	close_inet_socket_stream();
	free (ifr);

	if (hwsetup.mem_start)
	    return hwsetup.mem_start;
	else
	    return 0;

	return(EXIT_FALSE);
}


unsigned int get_interface_hw_memend(char *interface)
{
	ifr = (struct ifreq *) xmalloc (sizeof (struct ifreq));

	inet_socket = inet_socket_stream("get_interface_hw_memend()");

	strcpy (ifr->ifr_name, interface);
	if ((ioctl (inet_socket, SIOCGIFMAP, ifr)) < 0)
	    memset (&hwsetup, 0, sizeof(struct ifmap));
	else
	    memcpy (&hwsetup, &ifr->ifr_map, sizeof(struct ifmap));

	strcpy (ifr->ifr_name, interface);
	if ((ioctl (inet_socket, SIOCGIFMAP, ifr)) < 0)
	    memset (&hwsetup, 0, sizeof(struct ifmap));
	else
	    hwsetup = ifr->ifr_map;

	close_inet_socket_stream();
	free (ifr);

	if (hwsetup.mem_end)
	    return hwsetup.mem_end;
	else
	    return 0;

	return(EXIT_FALSE);
}


unsigned int get_interface_hw_base_addr(char *interface)
{
	ifr = (struct ifreq *) xmalloc (sizeof (struct ifreq));

	inet_socket = inet_socket_stream("get_interface_hw_base_addr()");

	strcpy (ifr->ifr_name, interface);
	if ((ioctl (inet_socket, SIOCGIFMAP, ifr)) < 0)
	    memset (&hwsetup, 0, sizeof(struct ifmap));
	else
	    memcpy (&hwsetup, &ifr->ifr_map, sizeof(struct ifmap));

	strcpy (ifr->ifr_name, interface);
	if ((ioctl (inet_socket, SIOCGIFMAP, ifr)) < 0)
	    memset (&hwsetup, 0, sizeof(struct ifmap));
	else
	    hwsetup = ifr->ifr_map;

	close_inet_socket_stream();
	free (ifr);

	if (hwsetup.base_addr)
	    return hwsetup.base_addr;
	else
	    return 0;

	return(EXIT_FALSE);
}


int get_interface_hw_irq(char *interface)
{
	ifr = (struct ifreq *) xmalloc (sizeof (struct ifreq));

	inet_socket = inet_socket_stream("get_interface_hw_irq()");

	strcpy (ifr->ifr_name, interface);
	if ((ioctl (inet_socket, SIOCGIFMAP, ifr)) < 0)
	    memset (&hwsetup, 0, sizeof(struct ifmap));
	else
	    memcpy (&hwsetup, &ifr->ifr_map, sizeof(struct ifmap));

	strcpy (ifr->ifr_name, interface);
	if ((ioctl (inet_socket, SIOCGIFMAP, ifr)) < 0)
	    memset (&hwsetup, 0, sizeof(struct ifmap));
	else
	    hwsetup = ifr->ifr_map;

	close_inet_socket_stream();
	free (ifr);

	if (hwsetup.irq)
	    return hwsetup.irq;
	else
	    return 0;

	return(EXIT_FALSE);
}


int get_interface_hw_dma(char *interface)
{
	ifr = (struct ifreq *) xmalloc (sizeof (struct ifreq));

	inet_socket = inet_socket_stream("get_interface_hw_dma()");

	strcpy (ifr->ifr_name, interface);
	if ((ioctl (inet_socket, SIOCGIFMAP, ifr)) < 0)
	    memset (&hwsetup, 0, sizeof(struct ifmap));
	else
	    memcpy (&hwsetup, &ifr->ifr_map, sizeof(struct ifmap));

	strcpy (ifr->ifr_name, interface);
	if ((ioctl (inet_socket, SIOCGIFMAP, ifr)) < 0)
	    memset (&hwsetup, 0, sizeof(struct ifmap));
	else
	    hwsetup = ifr->ifr_map;

	close_inet_socket_stream();
	free (ifr);

	if (hwsetup.dma)
	    return hwsetup.dma;
	else
	    return 0;

	return(EXIT_FALSE);
}


int get_interface_encap(char *interface)
{
	int en = 1, stat = 0;

	inet_socket = inet_socket_stream("get_interface_encap()");

	//if ((en = search_for_interface(interface)) == INTERFACE_NOT_FOUND)
	//    interface_not_found(interface);

	stat = (ioctl (inet_socket, SIOCGIFENCAP, &en));

	fprintf (stdout, "%d\n", stat);
	

	close_inet_socket_stream();
	
	if (stat != -1)
	    {
		fprintf (stdout, "%d\n", stat);
		return(EXIT_SUCCESS);
	    }

	return(EXIT_FALSE);
}



char* get_ethernet_bus_info(char *interface)
{
	ifr = (struct ifreq *) xmalloc (sizeof (struct ifreq));

	inet_socket = inet_socket_stream("get_ethernet_bus_info()");

	strcpy (ifr->ifr_name, interface);

	drv.cmd = ETHTOOL_GDRVINFO;
        ifr->ifr_data = (caddr_t) &drv;
    
        if (ioctl(inet_socket, SIOCETHTOOL, ifr) != -1)
	    {
		close_inet_socket_stream();
		free (ifr);

		return drv.bus_info;
	    }

	close_inet_socket_stream();
	free (ifr);

	return not_available;
}


char* get_ethernet_firmware(char *interface)
{
	ifr = (struct ifreq *) xmalloc (sizeof (struct ifreq));

	inet_socket = inet_socket_stream("get_ethernet_firmware()");

	strcpy (ifr->ifr_name, interface);

	drv.cmd = ETHTOOL_GDRVINFO;
        ifr->ifr_data = (caddr_t) &drv;
    
        if (ioctl(inet_socket, SIOCETHTOOL, ifr) != -1)
	    {
		close_inet_socket_stream();
		free (ifr);

		return drv.fw_version;
	    }

	close_inet_socket_stream();
	free (ifr);

	return not_available;
}


char* get_ethernet_driver_name(char *interface)
{
	ifr = (struct ifreq *) xmalloc (sizeof (struct ifreq));

	inet_socket = inet_socket_stream("get_ethernet_driver_name()");

	strcpy (ifr->ifr_name, interface);

	drv.cmd = ETHTOOL_GDRVINFO;
        ifr->ifr_data = (caddr_t) &drv;
    
        if (ioctl(inet_socket, SIOCETHTOOL, ifr) != -1)
	    {
		close_inet_socket_stream();
		free (ifr);

		return drv.driver;
	    }

	close_inet_socket_stream();
	free (ifr);

	return not_available;
}


char* get_ethernet_driver_version(char *interface)
{
	ifr = (struct ifreq *) xmalloc (sizeof (struct ifreq));

	inet_socket = inet_socket_stream("get_ethernet_driver_version()");

	strcpy (ifr->ifr_name, interface);

	drv.cmd = ETHTOOL_GDRVINFO;
        ifr->ifr_data = (caddr_t) &drv;

        if (ioctl(inet_socket, SIOCETHTOOL, ifr) != -1)
	    {
		close_inet_socket_stream();
		free (ifr);

		return drv.version;
	    }

	close_inet_socket_stream();
	free (ifr);

	return not_available;
}


char* get_ethernet_link_detect(char *interface)
{
	ifr = (struct ifreq *) xmalloc (sizeof (struct ifreq));

	inet_socket = inet_socket_stream("get_ethernet_link_detect()");

	strncpy(ifr->ifr_name, interface, sizeof(ifr->ifr_name) - 1);

	memset(&ifre, 0, sizeof (ifre));
	strncpy(ifre.ifr_name, interface, sizeof(ifre.ifr_name) - 1);

	val.cmd = ETHTOOL_GLINK;
        ifr->ifr_data = (caddr_t) &val;

        if (ioctl(inet_socket, SIOCETHTOOL, ifr) != -1)
	    {
		close_inet_socket_stream();
		free (ifr);

		return val.data ? STRING_YES : STRING_NO;
	    }
	else
	    free (ifr);

	if (ioctl(inet_socket, SIOCGMIIPHY, &ifre) != -1)
	    {
		((unsigned short *) &ifre.ifr_data)[1] = 1;

		if (ioctl(inet_socket, SIOCGMIIREG, &ifre) != -1)
		    {
			close_inet_socket_stream();

			return (((unsigned short *) &ifre.ifr_data)[3] & 0x0016) 
				== 0x0004 ? STRING_YES : STRING_NO;
		    }

	    }
	else if (ioctl(inet_socket, SIOCDEVPRIVATE, &ifre) != -1) 
	    {
		((unsigned short *) &ifre.ifr_data)[1] = 1;

		if (ioctl(inet_socket, SIOCDEVPRIVATE + 1, &ifre) != -1) 
		    {
			close_inet_socket_stream();

			return (((unsigned short *) &ifre.ifr_data)[3] & 0x0016) 
				== 0x0004 ? STRING_YES : STRING_NO;
		    }

	    }

	close_inet_socket_stream();

	return not_available;
}


char* get_ethernet_speed(char *interface)
{
	ifr = (struct ifreq *) xmalloc (sizeof (struct ifreq));

	inet_socket = inet_socket_stream("get_ethernet_speed()");

	strcpy (ifr->ifr_name, interface);

	ecmd.cmd = ETHTOOL_GSET;
        ifr->ifr_data = (caddr_t) &ecmd;

        if (ioctl(inet_socket, SIOCETHTOOL, ifr) != -1)
	    {
		close_inet_socket_stream();
		free (ifr);

		switch (ecmd.speed)
		    {
			case SPEED_10:
			    return TEN_MBYTES_PER_SEC;
			    break;
			case SPEED_100:
			    return HUNDRED_MBYTES_PER_SEC;
			    break;
			case SPEED_1000:
			    return THOUSAND_MBYTES_PER_SEC;
			    break;
			default:
			    return STRING_UNKNOWN;
			    break;
		    }
		
	    }

	close_inet_socket_stream();
	free (ifr);

	return not_available;
}


char* get_ethernet_duplex(char *interface)
{
	ifr = (struct ifreq *) xmalloc (sizeof (struct ifreq));

	inet_socket = inet_socket_stream("get_ethernet_duplex()");

	strcpy (ifr->ifr_name, interface);

	ecmd.cmd = ETHTOOL_GSET;
        ifr->ifr_data = (caddr_t) &ecmd;

        if (ioctl(inet_socket, SIOCETHTOOL, ifr) != -1)
	    {
		close_inet_socket_stream();
		free (ifr);

		switch (ecmd.duplex)
		    {
			case DUPLEX_HALF:
			    return ETHERNET_HALF_DUPLEX;
			    break;
			case DUPLEX_FULL:
			    return ETHERNET_FULL_DUPLEX;
			    break;
			default:
			    return STRING_UNKNOWN;
			    break;
		    }

	    }

	close_inet_socket_stream();
	free (ifr);

	return not_available;
}


/* If interface name was found then function return with 
    interface number which is installed in system (index) */
int search_for_interface(char *interface)
{
	int ic = 0, nr = 0;
	/*int flags = get_interface_flags(interface);*/

	/* If interface does not set UP flag then then 
	    will be display as `DN' (Down) */
	/*if (!(flags & IFF_UP))
	    return(INTERFACE_NOT_FOUND);*/
	
	ic = is->interfaces_counter;
	
	while (nr < ic)
	    {
		if (strcmp (is->interfaces_names[nr], interface) == 0)
		    {
			/* Return with interface number */
			return nr;
		    }

		nr++;
	    }
	    
	return(INTERFACE_NOT_FOUND);
}


unsigned int get_proc_file_size(char *file_patch)
{
	FILE *proc_file;
	unsigned int proc_file_size = 0;
	
	if ((proc_file = fopen (file_patch, "r")) == NULL)
	    return(EXIT_FALSE);
	    
	while (fgetc(proc_file) != EOF)
	    proc_file_size++;
	    
	fclose(proc_file);

	return proc_file_size;
}


void get_interface_stats(char *interface)
{
	FILE *file_fp;
	char ibuffer[get_proc_file_size(PROC_NET_DEV) + 2];
	char buffer_stats[STATSIZE], *iptr, interface_alias[IFNAMESIZE];
	int ilenght = 0, in = 0;
	ssize_t bytes_read;
		
	if (search_for_interface(interface) == INTERFACE_NOT_FOUND)
	    interface_not_found(interface);

	if ((file_fp = fopen (PROC_NET_DEV, "r")) == NULL)
	    devices_file_error();
	    
	bytes_read = fread (ibuffer, 1, sizeof(ibuffer), file_fp);
	fclose(file_fp);

	ibuffer[bytes_read] = '\0';

	/* Check if interface is an alias? */
	iptr = strstr (interface, ":");

	if (iptr != NULL)
	    {
		for (in = 0; in < (strlen(interface) - 2); in++)
		    interface_alias[in] = interface[in];
		
		interface_alias[(strlen(interface) - 2)] = '\0';
		
		interface = interface_alias;
	    }

	iptr = strstr (ibuffer, interface);

	/* ilenght = strlen(interface) */
	ilenght = strlen(interface);

	/* plus ':' */
	ilenght++;

	clear_buffer(buffer_stats);

	while (iptr[ilenght] != '\n')
	    {
		buffer_stats[ilenght - (strlen(interface) + 1)] = iptr[ilenght];

		ilenght++;
	    }

	buffer_stats[ilenght - (strlen(interface) + 1)] = '\0';

	sscanf (buffer_stats, 
	"%llu %llu %lu %lu %lu %lu %lu %lu %llu %llu %lu %lu %lu %lu %lu %lu", 
		&i->stats.rx_bytes, 
		&i->stats.rx_packets, 
		&i->stats.rx_errs, 
		&i->stats.rx_drop, 
		&i->stats.rx_fifo, 
		&i->stats.rx_frame,
		&i->stats.rx_compressed, 
		&i->stats.rx_multicast,
		&i->stats.tx_bytes, 
		&i->stats.tx_packets, 
		&i->stats.tx_errs, 
		&i->stats.tx_drop, 
		&i->stats.tx_fifo, 
		&i->stats.tx_colls, 
		&i->stats.tx_carrier, 
		&i->stats.tx_compressed);

	i->stats.rx_kbytes = ((double) i->stats.rx_bytes) / 1024;
	i->stats.rx_mbytes = i->stats.rx_kbytes / 1024;
	i->stats.tx_kbytes = ((double) i->stats.tx_bytes) / 1024;
	i->stats.tx_mbytes = i->stats.tx_kbytes / 1024;
}


/* Include threads functions header file */
#include "pthreads_func.h"


/* Reset structs interface speed */
void reset_interface_struct_stats (void)
{
	/* Packets TX/RX */
	i->speed.pss.last_tx_packets = 0;
	i->speed.pss.now_tx_packets = 0;
	i->speed.pss.last_rx_packets = 0;
	i->speed.pss.now_rx_packets = 0;
	i->speed.pss.speed_current_tx_packets = 0;
	i->speed.pss.speed_current_tx_packets_top = 0;
	i->speed.pss.speed_current_rx_packets = 0;
	i->speed.pss.speed_current_rx_packets_top = 0;
	i->speed.pss.speed_current_packets = 0;
	i->speed.pss.speed_packets_top = 0;

	/* KBytes TX/RX */
	i->speed.kbs.last_tx_kbytes = 0.0;
	i->speed.kbs.now_tx_kbytes = 0.0;
	i->speed.kbs.last_rx_kbytes = 0.0;
	i->speed.kbs.now_rx_kbytes = 0.0;
	i->speed.kbs.speed_current_tx_kbytes = 0.0;
	i->speed.kbs.speed_current_tx_kbytes_top = 0.0;
	i->speed.kbs.speed_current_rx_kbytes = 0.0;
	i->speed.kbs.speed_current_rx_kbytes_top = 0.0;
	i->speed.kbs.speed_current_kbytes = 0.0;
	i->speed.kbs.speed_kbytes_top = 0.0;
}


/* Thread read interface statistics after 1sec. */
void* interface_stats_read (void* unused)
{
	/* Sleep in sec. after filling struct interface_statistics *i */
	struct timespec tv;
	/*tv.tv_sec = 1;*/		/* 1 sec. */
	tv.tv_sec = 0;			/* 0 sec. */
	/*tv.tv_nsec = 500000;*/	/* 0.500 sec. */
	/*tv.tv_nsec = 50000;*/		/* 0.050 sec. */
	/*tv.tv_nsec = 40000;*/		/* 0.040 sec. */
	/*tv.tv_nsec = 30000;*/		/* 0.030 sec. */
	tv.tv_nsec = 25000;		/* 0.025 sec. */
	/*tv.tv_nsec = 20000;*/		/* 0.020 sec. */
	/*tv.tv_nsec = 10000;*/		/* 0.010 sec. */
	/*tv.tv_nsec = 0;*/		/* 0.000 sec. */

	while (shutdown_thread_ife_stats == FALSE)
		{
		    pthread_mutex_lock (&mutex_id);

		    get_interface_stats(i->name);

#ifdef _BCOUNT_H_
		    if (shutdown_thread_ife_stats == FALSE)
			activate_leds();
#endif
		    i->stats.tx_packets_last = i->stats.tx_packets;
		    i->stats.rx_packets_last = i->stats.rx_packets;

		    pthread_mutex_unlock (&mutex_id);

		    /* Sleeping tv.tv_sec & tv.tv_nsec */
		    if ((nanosleep(&tv, &tv)) == EINTR)
			continue;
#ifdef _BCOUNT_H_
		    pthread_mutex_lock (&mutex_id);

		    if (shutdown_thread_ife_stats == FALSE)
			deactivate_leds();

		    pthread_mutex_unlock (&mutex_id);

		    /* Sleeping tv.tv_sec & tv.tv_nsec */
		    if ((nanosleep(&tv, &tv)) == EINTR)
			continue;
#endif
		}		    

	/* Do this if shutdown_thread_ife_stats flag is TRUE */
	return(EXIT_SUCCESS);
}


/* Thread measuring speed on an interface */
void* interface_speed_measure (void* unused)
{
	pthread_mutex_lock (&mutex_id);

	/* Reset structs interface speed before enter in while() */
	reset_interface_struct_stats();

	pthread_mutex_unlock (&mutex_id);

	while (shutdown_thread_ife_speed == FALSE)
		{
		    pthread_mutex_lock (&mutex_id);

		    if (clear_interface_speed_flag == TRUE)
			{
			    reset_interface_struct_stats();
#ifdef _BCOUNT_H_
			    reset_speed_window();
#endif
			    clear_interface_speed_flag = FALSE; /* reset flag */
			}

		    /* Packets */
		    /* TX current = stats.tx_packets 
			and RX current = stats.rx_packets */
		    i->speed.pss.now_tx_packets = i->stats.tx_packets;
		    i->speed.pss.now_rx_packets = i->stats.rx_packets;

		    /* (Packets) If TX last and RX last is 0.0 then 
			TX last = TX current and RX last = RX current  */
		    if (i->speed.pss.last_tx_packets == 0 && 
			i->speed.pss.last_rx_packets == 0)
			{
			    i->speed.pss.last_tx_packets = i->speed.pss.now_tx_packets;
			    i->speed.pss.last_rx_packets = i->speed.pss.now_rx_packets;
			}
		    
		    /* (Packets) Speed TX current */
		    i->speed.pss.speed_current_tx_packets =
			    (i->speed.pss.now_tx_packets - i->speed.pss.last_tx_packets); 

		    /* (Packets) Speed RX current */
		    i->speed.pss.speed_current_rx_packets =
			    (i->speed.pss.now_rx_packets - i->speed.pss.last_rx_packets);

		    /* (Packets) Speed current = (TX + RX) current */
		    i->speed.pss.speed_current_packets = 
			    ((i->speed.pss.now_tx_packets - i->speed.pss.last_tx_packets) + 
			    (i->speed.pss.now_rx_packets - i->speed.pss.last_rx_packets));

		    /* (Packets) If TX current is > 
			TX top then TX top = TX current */
		    if (i->speed.pss.speed_current_tx_packets > 
			i->speed.pss.speed_current_tx_packets_top)
			{
			    i->speed.pss.speed_current_tx_packets_top =
			    i->speed.pss.speed_current_tx_packets;
			}

		    /* (Packets) If RX current is > 
			RX top then RX top = RX current */
		    if (i->speed.pss.speed_current_rx_packets > 
			i->speed.pss.speed_current_rx_packets_top)
			{
			    i->speed.pss.speed_current_rx_packets_top = 
			    i->speed.pss.speed_current_rx_packets;
			}

		    /* (Packets) Speed current = (TX + RX) > 
			Speed last = (TX + RX) then Speed last  = Speed current */
		    if (i->speed.pss.speed_current_packets > i->speed.pss.speed_packets_top)
			i->speed.pss.speed_packets_top = i->speed.pss.speed_current_packets;

		    /* (Packets) TX last = TX current and RX last = RX current  */
		    i->speed.pss.last_tx_packets = i->speed.pss.now_tx_packets;
		    i->speed.pss.last_rx_packets = i->speed.pss.now_rx_packets;
	

		    /* KBytes */
		    /* TX current = stats.tx_kbytes 
			and RX current = stats.rx_kbytes */
		    i->speed.kbs.now_tx_kbytes = i->stats.tx_kbytes;
		    i->speed.kbs.now_rx_kbytes = i->stats.rx_kbytes;

		    /* If TX last and RX last is 0.0 then TX last = TX current 
			and RX last = RX current  */
		    if (i->speed.kbs.last_tx_kbytes == 0.0 && 
			i->speed.kbs.last_rx_kbytes == 0.0)
			{
			    i->speed.kbs.last_tx_kbytes = i->speed.kbs.now_tx_kbytes;
			    i->speed.kbs.last_rx_kbytes = i->speed.kbs.now_rx_kbytes;
			}
		    
		    /* Speed TX current */
		    i->speed.kbs.speed_current_tx_kbytes =
			    (i->speed.kbs.now_tx_kbytes - i->speed.kbs.last_tx_kbytes); 

		    /* Speed RX current */
		    i->speed.kbs.speed_current_rx_kbytes =
			    (i->speed.kbs.now_rx_kbytes - i->speed.kbs.last_rx_kbytes);

		    /* Speed current = (TX + RX) current */
		    i->speed.kbs.speed_current_kbytes = 
			    ((i->speed.kbs.now_tx_kbytes - i->speed.kbs.last_tx_kbytes) + 
			    (i->speed.kbs.now_rx_kbytes - i->speed.kbs.last_rx_kbytes));

		    /* If TX current is > TX top then TX top = TX current */
		    if (i->speed.kbs.speed_current_tx_kbytes > 
			i->speed.kbs.speed_current_tx_kbytes_top)
			{
			    i->speed.kbs.speed_current_tx_kbytes_top =
			    i->speed.kbs.speed_current_tx_kbytes;
			}

		    /* If RX current is > RX top then RX top = RX current */
		    if (i->speed.kbs.speed_current_rx_kbytes > 
			i->speed.kbs.speed_current_rx_kbytes_top)
			{
			    i->speed.kbs.speed_current_rx_kbytes_top = 
			    i->speed.kbs.speed_current_rx_kbytes;
			}

		    /* Speed current = (TX + RX) > Speed last = (TX + RX)
			then Speed last  = Speed current */
		    if (i->speed.kbs.speed_current_kbytes > i->speed.kbs.speed_kbytes_top)
			i->speed.kbs.speed_kbytes_top = i->speed.kbs.speed_current_kbytes;

		    /* TX last = TX current and RX last = RX current  */
		    i->speed.kbs.last_tx_kbytes = i->speed.kbs.now_tx_kbytes;
		    i->speed.kbs.last_rx_kbytes = i->speed.kbs.now_rx_kbytes;
	
		    pthread_mutex_unlock (&mutex_id);
		    
		    /* Sleeping SLEEP_TIME then run */
		    sleep(SLEEP_TIME);
		}

	/* Do this if shutdown_thread_ife_speed flag is TRUE */
	return(EXIT_SUCCESS);
}


/* NIN */

#endif /* _INTERFACE_H_ */


