#ifndef _INTERFACE_DESC_H_
#define _INTERFACE_DESC_H_

/*
   *
   * interface_desc.h
   *
   * Copyright (c) 2002-2003 Aleksander Augustowski
   *
   * BUG RAPORTS &
   * E-MAIL:          aleksander.augustowski@ins.com.pl
   * URL:             http://www.ins.com.pl/~alex/bcount/bcount.html
   *
   * This software is open source; you can redistribute it and/or modify
   * it under the terms of the GNU General Public License as published by
   * the Free Software Foundation; either version 2 of the License, or
   * (at your option) any later version.
   *
   * This program is distributed WITHOUT ANY WARRANTY; without even the
   * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
   * See the GNU General Public License in the included COPYING file for
   * details.
   *
   *
*/



#define DESC_UNKNOWN			"Unknown"
#define DESC_UNSPEC			"Unspec"
#define DESC_LOOPBACK			"Loopback"
#define DESC_ETHERNET			"Ethernet"
#define DESC_IPV6INIPV4			"IPv6 in IPv4"
#define DESC_PPP			"Point to point"
#define DESC_IPPP			DESC_PPP
#define DESC_SLIP			"Serial Line IP"
#define DESC_TOKENRING			"16/4 Token Ring"
#define DESC_FDDI			"Fiber Dist Data"
#define DESC_ARCNET			"2.5Mbps ARCnet"



/* This is 'inet' interfaces definition only */
char* get_interface_desc(char *ife)
{
	if (isdigit (ife[strlen(ife) - 1]) != 0)
	    {
		/* Ethernet "lo:x" */
		if ((strncmp (ife, LOOPBACK, strlen(ife) )) == 0
		    || (((strncmp (ife, LOOPBACK, strlen(ife) - 2)) == 0
		    && strstr(ife, ":") != NULL)))
		    return DESC_LOOPBACK;
		/* Ethernet "ethx" and "ethx:x" */
		if ((strncmp (ife, ETHERNET, strlen(ife) - 1)) == 0
		    || (((strncmp (ife, ETHERNET, strlen(ife) - 3)) == 0
		    && strstr(ife, ":") != NULL)))
		    return DESC_ETHERNET;
		/* Token Ring "trx" and "trx:x" */
		else if ((strncmp (ife, TOKEN_RING, strlen(ife) - 1)) == 0
		    || (((strncmp (ife, TOKEN_RING, strlen(ife) - 3)) == 0
		    && strstr(ife, ":") != NULL)))
		    return DESC_TOKENRING;
		/* 6BONE "sitx" and "sitx:x" */
		else if ((strncmp (ife, IPV6_IN_IPV4, strlen(ife) - 1)) == 0
		    || (((strncmp (ife, IPV6_IN_IPV4, strlen(ife) - 3)) == 0
		    && strstr(ife, ":") != NULL)))
		    return DESC_IPV6INIPV4;
		/* MoDem Point to point Protocol "pppx" and "pppx:x" */
		else if ((strncmp (ife, MODEM_PPP, strlen(ife) - 1)) == 0
		    || (((strncmp (ife, MODEM_PPP, strlen(ife) - 3)) == 0
		    && strstr(ife, ":") != NULL)))
		    return DESC_PPP;
		/* ISDN Point to point Protocol "ipppx" and "ipppx:x" */
		else if ((strncmp (ife, ISDN_PPP, strlen(ife) - 1)) == 0
		    || (((strncmp (ife, ISDN_PPP, strlen(ife) - 3)) == 0
		    && strstr(ife, ":") != NULL)))
		    return DESC_IPPP;
		/* Serial Line "slx" and "slx:x" */
		else if ((strncmp (ife, SERIAL_LINE, strlen(ife) - 1)) == 0
		    || (((strncmp (ife, SERIAL_LINE, strlen(ife) - 3)) == 0
		    && strstr(ife, ":") != NULL)))
		    return DESC_SLIP;
		/* Fiber Distributed Data (FDDI) "fddix" and "fddix:x" */
		else if ((strncmp (ife, FDDI, strlen(ife) - 1)) == 0
		    || (((strncmp (ife, FDDI, strlen(ife) - 3)) == 0
		    && strstr(ife, ":") != NULL)))
		    return DESC_FDDI;
		/* ARCnet "arcx" and "arcx:x" */
		else if ((strncmp (ife, ARCNET, strlen(ife) - 1)) == 0
		    || (((strncmp (ife, ARCNET, strlen(ife) - 3)) == 0
		    && strstr(ife, ":") != NULL)))
		    return DESC_ARCNET;
		/* Unknown */
		else
		    return DESC_UNKNOWN;
	    }
	else
	    {
		/* Local Loopback "lo" */
		if ((strcmp (ife, LOOPBACK)) == 0)
		    return DESC_LOOPBACK;
		/* Unspecified "un" FIXME: That is name for unspecified ife? */
		else if ((strcmp (ife, UNSPEC)) == 0)
		    return DESC_UNSPEC;
		/* Unspecified "unspec" 
		    FIXME: That is name for unspecified ife? */
		else if ((strcmp (ife, UNSPEC_FIX)) == 0)
		    return DESC_UNSPEC;
		/* Unknown */
		else
		    return DESC_UNKNOWN;
	    }

	/* Return with unknown interface description */
	return DESC_UNKNOWN;
}


/* NIN */

#endif /* _INTERFACE_DESC_H_ */

