#ifndef _SOCKET_STREAM_H_
#define _SOCKET_STREAM_H_

/*
   *
   * socket_stream.h
   *
   * Copyright (c) 2002-2003 Aleksander Augustowski
   *
   * BUG RAPORTS &
   * E-MAIL:          aleksander.augustowski@ins.com.pl
   * URL:             http://www.ins.com.pl/~alex/bcount/bcount.html
   *
   * This software is open source; you can redistribute it and/or modify
   * it under the terms of the GNU General Public License as published by
   * the Free Software Foundation; either version 2 of the License, or
   * (at your option) any later version.
   *
   * This program is distributed WITHOUT ANY WARRANTY; without even the
   * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
   * See the GNU General Public License in the included COPYING file for
   * details.
   *
   *
*/


#include <sys/ioctl.h>
#include <sys/socket.h>
#include "socket.h"
#include "pf_af_inet.h"		/* Be sure PF_INET and AF_INET is defined! */


int inet_socket = -1;
int tcp_socket = -1;


void close_inet_socket_stream(void)
{
	close(inet_socket);
	close(tcp_socket);
}


int inet_socket_stream(char *function)
{
	if ((tcp_socket = socket (PF_INET, SOCK_STREAM, TCP_PROTOCOL)) < 0)
	    {
		fprintf (stderr, _STRING_RED_": inet_socket_stream() -> "
			"socket() called with %s.\n", strerror(errno), function);
		
		if (errno == EPROTONOSUPPORT) {
		    fprintf (stderr, "Error number ["_COLOR_WHITE_"%d"
			    _COLOR_END_" -> EPROTONOSUPPORT].\nThe protocol "
			    "type or the specified protocol\nis not supported"
			    " within this domain.\nWhy got this error, "
			    "because we use\nPF_INET aka AF_INET socket "
			    "family!?\n", errno);
		    }
		else if (errno == EAFNOSUPPORT) {
		    fprintf (stderr, "Error number ["_COLOR_WHITE_"%d"
			    _COLOR_END_" -> EAFNOSUPPORT].\nThe implementation "
			    " does not support\nthe specified address family.\n", 
			    errno);
		    }
		else if (errno == ENFILE) {
		    fprintf (stderr, "Error number ["_COLOR_WHITE_"%d"
			    _COLOR_END_" -> ENFILE].\nNot enough kernel memory "
			    "to allocate\na new socket structure.\n", errno);
		    }
		else if (errno == EMFILE) {
		    fprintf (stderr, "Error number ["_COLOR_WHITE_"%d"
			    _COLOR_END_" -> EMFILE].\nProcess file table "
			    "overflow.\n", errno);
		    }
		else if (errno == EACCES) {
		    fprintf (stderr, "Error number ["_COLOR_WHITE_"%d"
			    _COLOR_END_" -> EACCES].\nPermission to create "
			    "a socket of the specified\ntype and/or "
			    "protocol is denied.\n", errno);
		    }
		else if (errno == ENOBUFS || errno == ENOMEM) {
		    fprintf (stderr, "Error number ["_COLOR_WHITE_"%d"
			    _COLOR_END_" -> ENOBUFS/ENOMEM].\nInsufficient "
			    "memory is available. The socket canot be\n"
			    "created until sufficient resources are freed.\n", 
			    errno);
		    }
		else if (errno == EINVAL) {
		    fprintf (stderr, "Error number ["_COLOR_WHITE_"%d"
			    _COLOR_END_" -> EINVAL].\nUnknown protocol, or "
			    "protocol family not available.\n", errno);
		    }
		else 
		    {
			fprintf (stderr, "Error number ["_COLOR_WHITE_"%d"
			    _COLOR_END_" -> ?].\nUnknown! :-(. Whats is wrong\n"
			    "Mr. Nice guy! ;-).\n", errno);
		    }

#ifndef _SMTP_CLIENT_H_
		exit_prog(EXIT_FALSE);
#endif
	    }

	return tcp_socket;
}




/* NIN */

#endif /* _SOCKET_STREAM_H_ */

