#ifndef _BCOUNT_H_
#define _BCOUNT_H_

/*
   *
   * bcount.h
   *
   * Copyright (c) 2002-2003 Aleksander Augustowski
   *
   * BUG RAPORTS &
   * E-MAIL:          aleksander.augustowski@ins.com.pl
   * URL:             http://www.ins.com.pl/~alex/bcount/bcount.html
   *
   * This software is open source; you can redistribute it and/or modify
   * it under the terms of the GNU General Public License as published by
   * the Free Software Foundation; either version 2 of the License, or
   * (at your option) any later version.
   *
   * This program is distributed WITHOUT ANY WARRANTY; without even the
   * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
   * See the GNU General Public License in the included COPYING file for
   * details.
   *
   *
*/


#include <sys/stat.h>
#include <curses.h>
#include <fcntl.h>
#include <sys/time.h>
#include <pwd.h>
#include <locale.h>
#include "programs_version.h"
#include "constans.h"
#include "includes.h"
#include "bugy_function.h"
#include "colours.h"
#include "colours_curses.h"
#include "xmalloc.h"
#include "host_info.h"
#include "file_type.h"
#include "author_info.h"
#include "print_info.h"
#include "data_type.h"
#include "sys_info.h"
#include "host_name.h"
#include "interface.h"
#include "press_any_key.h"
#include "date_time.h"
#include "smtp_client.h"
#include "tempfile.h"
#include "terminal_setting.h"


#define KEY_BKSP	127             /* BackSpace Key */
#define KEY_ESC		27              /* Escape Key */

#define PROC_UPTIME	"/proc/uptime"  /* Up time :-) */

#define SYS_REBOOT	"Reboot"
#define SYS_HALT	"Halt"
#define DEFAULT_WAIT	5


WINDOW *main_window;
WINDOW *void_window;
WINDOW *about_window;
WINDOW *info_window;
WINDOW *interface_window;
WINDOW *devices_window;
WINDOW *now_window;
WINDOW *other_data_window;
WINDOW *total_window;
WINDOW *search_window;
WINDOW *mail_window;
WINDOW *down_window;
WINDOW *status_window;
WINDOW *speed_window;
WINDOW *time_window;
WINDOW *sys_info_window;
WINDOW *choice_if_window;
WINDOW *interface_info_window;
WINDOW *quit_window;
WINDOW *quit_window_sh;


void print_usage(void) __attribute__ ((noreturn));
void read_log_error(void) __attribute__ ((noreturn));
void write_log_error(void) __attribute__ ((noreturn));

void init_ncurses_screen(void);
void draw_ncurses_screen(void);
void close_ncurses_screen(void);
void make_ncurses_void_window(void);
void make_ncurses_about_window(void);
void make_ncurses_threads_sync_window(void);
void syncing_and_down_threads(void);
void make_ncurses_quit_window(void);
void make_ncurses_total_window(void);
void make_ncurses_speed_window(void);
void make_ncurses_status_window(void);
void make_ncurses_mail_status_window(void);
void make_ncurses_time_window(void);
void make_ncurses_sys_info_window(void);
void make_ncurses_choice_if_window(void);
void make_ncurses_interface_info_window(void);
void make_ncurses_interface_window(void);
void make_ncurses_search_window(void);
void make_ncurses_mail_window(void);
void make_ncurses_down_window(void);
void make_ncurses_info_window(void);
void make_ncurses_devices_window(void);
void make_ncurses_now_window(void);
void make_ncurses_other_data_window(void);
void refresh_memory_status(void);
void refresh_rss_memory_status(void);
void refresh_local_time_status(void);
void refresh_other_data_window(void);
void clrscr(void);
int get_file_info (char *filename);
/*void* refresh_now_window(void* unused);*/ /* declared in pthreads.h */
int searching(void);
int total_bytes(void);
int get_nr_record(void);
int summary_bytes(void);
int read_log(void)/* __attribute__ ((noreturn))*/;
int write_log_as_text(void)/* __attribute__ ((noreturn))*/;
int write_log(void) __attribute__ ((noreturn));
void log_file_older_than_ver4_msg(void);
int check_if_old_log(void);
void check_log_read(char *file_for_read);
void check_log_write(char *file_for_write);
int choice_interface(void);
int get_system_info(void);
int make_mail(void);
void make_ncurses_shutdown_window(char *reboot_or_halt, int wait);
void shutdown_system(char *reboot_or_halt, int wait);


struct log_records_old
{
	char data[11];
	char interface[20];
	unsigned int rx;
	unsigned int tx;
	int record;
};

struct log_records_old log_old;


struct log_records
{
	char data[11];
	char interface[20];
	unsigned long long rx;
	unsigned long long tx;
	unsigned long record;
};

struct log_records log;


struct option longopts[] = {
	{"interface",       1, 0, 'i'},
	{"logfile",         1, 0, 'f'},
	{"data",            1, 0, 'd'},
	{"smtp",            1, 0, 'm'},
	{"smtp_file",       1, 0, 'n'},
	{"smtp_debug",      0, 0, 'p'},
	{"leds_off",        0, 0, 'o'},
	{"writelog",        0, 0, 'w'},
	{"readlog",         0, 0, 'r'},
	{"text_output",     0, 0, 't'},
	{"summary",         0, 0, 's'},
	{"signals",         0, 0, 'g'},
	{"version",         0, 0, 'v'},
	{"license",         0, 0, 'l'},
	{"help",            0, 0, 'h'},
	{ 0, 0, 0, 0 }
};


/* Log file veriables */
char default_log_file[100] = "/var/log/bcount.log";
char *log_file = default_log_file;
int log_file_size = 0, file_uid = 0, file_gid = 0;
int record_number = 0;
unsigned short int data_is_available = FALSE;

/* Other veriables */
int counter = 19;
int y_max = 0, x_max = 0;
int now_y_max = 0, now_x_max = 0;
struct timespec tvn;
char get_down_digit[25];

/* check if `q', `Q', `r', `R', LN or `KEY_ESC' is not pressed */
char received_char_check;
/* check if `q', `Q', `t', `T', LN or `KEY_ESC' is not pressed */
char transmitted_char_check;		
/* check if `q', `Q', `r', `R', LN or `KEY_ESC' is not pressed */
char reboot_char_check;		
/* check if `q', `Q', `s', `S', LN or `KEY_ESC' is not pressed */
char shutdown_char_check;		
/* TRUE = turn leds on, FALSE = turn leds off */
int activity_leds_flag = TRUE;			
/* TRUE = refresh WIDNOW *info_window */
int active_window_info_flag = FALSE;		
/* TRUE = refresh WIDNOW *other_data_window */
int active_window_other_data_flag = FALSE;	
/* TRUE = refresh WIDNOW *interface_info_window */
int active_window_interface_info_flag = FALSE;	
/* TRUE = refresh WIDNOW *devices_window */
int active_window_devices_flag = FALSE;	
/* TRUE = reboot the system */
int reboot_system_flag = FALSE;	
/* TRUE = shutdown the system */
int shutdown_system_flag = FALSE;	



void print_version(void)
{
	fprintf (stdout, "BYTES COUNTER v"_STRING_WHITE_, VERSION);
}


void print_welcome(void)
{
	fprintf (stdout, "-== ");
	print_version();
	fprintf (stdout, " on an %s running... ==-\n", 
		get_local_host_info(HOST_ARCHITECTURE));

	free(u_name);
}


void print_usage(void)
{
	fprintf (stdout, "Usage: %s [OPTIONS] ...", exec_name);
	fprintf (stdout, "\nExample: %s -i ppp0            "
		"<- Reading bytes for ppp0 in interactive mode\n         "
		"%s -i ppp0 --readlog  <- Reading bytes for ppp0 from "
		"log file\n         "
		"If you don't choice interface then default interface will "
		"be %s!\n"
		"Options:\n"
		"  -i [INTERFACE]  --interface   "
		"Choice Interface (default: %s)\n"
		"  -f [PATH]       --logfile     "
		"Path for log file (default: %s)\n"
		"  -d [DATA TYPE]  --data	"
		"Choice what we want get (MB<->KB<->B)\n"
		"  -m [SERVER]     --smtp        "
		"Remote SMTP server name.\n"
		"  -p		  --smtp_debug  Write SMTP session to debug file\n"
		"  -n [SMTP FILE]  --smtp_file   Path for SMTP session debug file\n"
		"				(default: %s)\n"
		"  -o      	  --leds_off    Don't blinking any fuc*ing leds\n"
		"  -w      	  --writelog    Write to log file\n"
		"  -r      	  --readlog     Read from log file\n"
		"  -t      	  --text_output Write to file stats from log file\n"
		"  -s      	  --summary     Summary bytes from log file\n"
		"  -g      	  --signals     See information about signals\n"
		"  -l		  --license     Print license for this program\n"
		"  -v, -V  	  --version     Print program version\n"
		"  -h      	  --help        Print short help\n\n"
		"Please run `do_white.sh' if you can't see white digits "
		"(underline)\n\n",
		exec_name, exec_name, default_interface_name, default_interface_name, 
		default_log_file, default_smtp_debug_file);

	exit_prog(EXIT_SUCCESS);
}


void read_log_error(void)
{
	fprintf (stderr, _STRING_RED_": File "_STRING_WHITE_" not found!, "
		"exiting(1)...\n", strerror(errno), log_file);

	exit_prog(EXIT_FALSE);
}


void write_log_error(void)
{
	fprintf (stderr, _STRING_RED_": You don't have access to write in\n"
		_STRING_WHITE_", exiting(1)...\n", strerror(errno), log_file);

	exit_prog(EXIT_FALSE);
}


void copyright_info(int exit_code)
{
	fprintf (stdout, "\t    Copyright (c) 2002-2003 by %s\n"
		"\t\t\t    Web: %s\n\t\t\t    E-mail: %s\n"_COLOR_WHITE_
		"Exiting(%d) ..."_COLOR_END_"\n", author_name, page_address, 
		email_address, exit_code);
}


void shutdown_system(char *reboot_or_halt, int wait)
{
	syncing_and_down_threads();

	make_ncurses_shutdown_window(reboot_or_halt, wait);

	if (curses_flag == TRUE)
	    close_ncurses_screen();

	clrscr();
	
	if (!strcmp (reboot_or_halt, SYS_REBOOT))
	    {
		fprintf (stdout, "Shutdown and reboot . . .\n");
		fflush(stdout);
		//execl ("/bin/sh", "sh", "-c", "shutdown -r now", char *)0);
	    }
	else
	    {
		fprintf (stdout, "Shutdown and halt . . .\n");
		fflush(stdout);
		//execl ("/bin/sh", "sh", "-c", "shutdown -h now", (char *)0);
	    }

	exit(0);
}


/* Exit confirmation */
void quit_confirmation(void)
{
	int *c = xmalloc (sizeof(int));
	
	pthread_mutex_lock(&mutex_id);
	
	quit_flag = TRUE;
	curs_set(INVISIBLE);
	make_ncurses_quit_window();

	switch_go2:
	switch (*c = getch())
	    {
		case 'y':
		case 'Y':
		    free(c);

		    pthread_mutex_unlock(&mutex_id);
		    
		    syncing_and_down_threads();
		    
		    if (curses_flag == TRUE)
			close_ncurses_screen();

		    clrscr();
		    print_version();
		    copyright_info(0);
		    print_process_stats();
		    fprintf (stdout, "\n");
		    fflush (stdout);
		    fflush (stderr);
		    		    
		    exit_prog(EXIT_SUCCESS);
		    break;
		case 'n':
		case 'N':
		    free(c);
		    quit_flag = FALSE;
		    draw_ncurses_screen();
		    if (searching_flag == TRUE)
			{
			    refresh_local_time_status();
			    refresh_rss_memory_status();
			    refresh_memory_status();
			    get_system_info();
			    make_ncurses_search_window();
			    move (4, 58);
			    wmove (search_window, 2, 20);
			    wattron (search_window, A_NORMAL | COLOR_PAIR(2));
			    wrefresh (search_window);
			    curs_set(VISIBLE);
			}
		    else if (email_flag == TRUE)
			{
			    refresh_local_time_status();
			    refresh_rss_memory_status();
			    refresh_memory_status();
			    get_system_info();
			    make_ncurses_mail_window();
			    move (4, 56);
			    wmove (mail_window, 2, 18);
			    wattron (mail_window, A_NORMAL | COLOR_PAIR(2));
			    wrefresh(mail_window);
			}
		    else
			{
			    refresh_local_time_status();
			    refresh_rss_memory_status();
			    refresh_memory_status();
			    get_system_info();
			}
		    pthread_mutex_unlock(&mutex_id);
		    break;
		default:
		    goto switch_go2;
		    break;
	    }
	
}



/* NIN */

#endif /* _BCOUNT_H_ */

